@@ -1,4 +1,4 @@
-/* $Id: VBoxServiceVMInfo-win.cpp 38131 2011-07-25 08:38:59Z vboxsync $ */
+/* $Id: VBoxServiceVMInfo-win.cpp $ */
 /** @file
  * VBoxService - Virtual Machine Information for the Host, Windows specifics.
  */
@@ -49,6 +49,8 @@ typedef struct
     WCHAR wszUser[_MAX_PATH];
     WCHAR wszAuthenticationPackage[_MAX_PATH];
     WCHAR wszLogonDomain[_MAX_PATH];
+    /** Number of assigned user processes. */
+    ULONG ulNumProcs;
 } VBOXSERVICEVMINFOUSER, *PVBOXSERVICEVMINFOUSER;
 
 /** Structure for the file information lookup. */
@@ -69,7 +71,7 @@ typedef struct
 /*******************************************************************************
 *   Prototypes
 *******************************************************************************/
-bool VBoxServiceVMInfoWinSessionHasProcesses(PLUID pSession, VBOXSERVICEVMINFOPROC const *paProcs, DWORD cProcs);
+uint32_t VBoxServiceVMInfoWinSessionHasProcesses(PLUID pSession, VBOXSERVICEVMINFOPROC const *paProcs, DWORD cProcs);
 bool VBoxServiceVMInfoWinIsLoggedIn(PVBOXSERVICEVMINFOUSER a_pUserInfo, PLUID a_pSession);
 int  VBoxServiceVMInfoWinProcessesEnumerate(PVBOXSERVICEVMINFOPROC *ppProc, DWORD *pdwCount);
 void VBoxServiceVMInfoWinProcessesFree(PVBOXSERVICEVMINFOPROC paProcs);
@@ -248,17 +250,17 @@ void VBoxServiceVMInfoWinProcessesFree(PVBOXSERVICEVMINFOPROC paProcs)
 /**
  * Determines whether the specified session has processes on the system.
  *
- * @returns true if it has, false if it doesn't.
+ * @returns Number of processes found for a specified session.
  * @param   pSession        The session.
  * @param   paProcs         The process snapshot.
  * @param   cProcs          The number of processes in the snaphot.
  */
-bool VBoxServiceVMInfoWinSessionHasProcesses(PLUID pSession, VBOXSERVICEVMINFOPROC const *paProcs, DWORD cProcs)
+uint32_t VBoxServiceVMInfoWinSessionHasProcesses(PLUID pSession, VBOXSERVICEVMINFOPROC const *paProcs, DWORD cProcs)
 {
     if (!pSession)
     {
         VBoxServiceVerbose(1, "VMInfo/Users: Session became invalid while enumerating!\n");
-        return false;
+        return 0;
     }
 
     PSECURITY_LOGON_SESSION_DATA pSessionData = NULL;
@@ -266,7 +268,7 @@ bool VBoxServiceVMInfoWinSessionHasProcesses(PLUID pSession, VBOXSERVICEVMINFOPR
     if (rcNt != STATUS_SUCCESS)
     {
         VBoxServiceError("VMInfo/Users: Could not get logon session data! rcNt=%#x", rcNt);
-        return false;
+        return 0;
     }
 
     /*
@@ -289,11 +291,15 @@ bool VBoxServiceVMInfoWinSessionHasProcesses(PLUID pSession, VBOXSERVICEVMINFOPR
         }
     }
 
-    VBoxServiceVerbose(3, "VMInfo/Users: Session %u has %u processes\n",
-                       pSessionData->Session, cNumProcs);
+    if (g_cVerbosity >= 4)
+        VBoxServiceVerbose(3, "VMInfo/Users: Session %u has %u processes\n",
+                           pSessionData->Session, cNumProcs);
+    else
+        VBoxServiceVerbose(3, "VMInfo/Users: Session %u has at least one process\n",
+                           pSessionData->Session);
 
     LsaFreeReturnBuffer(pSessionData);
-    return cNumProcs ? true : false;
+    return cNumProcs;
 }
 
 
@@ -379,11 +385,12 @@ bool VBoxServiceVMInfoWinIsLoggedIn(PVBOXSERVICEVMINFOUSER pUserInfo, PLUID pSes
             || (SECURITY_LOGON_TYPE)pSessionData->LogonType == CachedInteractive
             || (SECURITY_LOGON_TYPE)pSessionData->LogonType == CachedRemoteInteractive))
     {
-        VBoxServiceVerbose(3, "VMInfo/Users: Session data: Name=%ls, Len=%d, SID=%s, LogonID=%ld,%ld\n",
+        VBoxServiceVerbose(3, "VMInfo/Users: Session data: Name=%ls, Len=%d, SID=%s, LogonID=%ld,%ld, LogonType=%ld\n",
                            pSessionData->UserName.Buffer,
                            pSessionData->UserName.Length,
                            pSessionData->Sid != NULL ? "1" : "0",
-                           pSessionData->LogonId.HighPart, pSessionData->LogonId.LowPart);
+                           pSessionData->LogonId.HighPart, pSessionData->LogonId.LowPart,
+                           pSessionData->LogonType);
 
         /*
          * Copy out relevant data.
@@ -502,18 +509,18 @@ bool VBoxServiceVMInfoWinIsLoggedIn(PVBOXSERVICEVMINFOUSER pUserInfo, PLUID pSes
 int VBoxServiceVMInfoWinWriteUsers(char **ppszUserList, uint32_t *pcUsersInList)
 {
     PLUID       paSessions = NULL;
-    ULONG       cSession = 0;
+    ULONG       cSessions = 0;
 
     /* This function can report stale or orphaned interactive logon sessions
        of already logged off users (especially in Windows 2000). */
-    NTSTATUS rcNt = LsaEnumerateLogonSessions(&cSession, &paSessions);
+    NTSTATUS rcNt = LsaEnumerateLogonSessions(&cSessions, &paSessions);
     if (rcNt != STATUS_SUCCESS)
     {
         ULONG ulError = LsaNtStatusToWinError(rcNt);
         switch (ulError)
         {
             case ERROR_NOT_ENOUGH_MEMORY:
-                VBoxServiceVerbose(3, "VMInfo/Users: Not enough memory to enumerate logon sessions!\n");
+                VBoxServiceError("VMInfo/Users: Not enough memory to enumerate logon sessions!\n");
                 break;
 
             case ERROR_SHUTDOWN_IN_PROGRESS:
@@ -530,7 +537,7 @@ int VBoxServiceVMInfoWinWriteUsers(char **ppszUserList, uint32_t *pcUsersInList)
 
         return RTErrConvertFromWin32(ulError);
     }
-    VBoxServiceVerbose(3, "VMInfo/Users: Found %ld sessions\n", cSession);
+    VBoxServiceVerbose(3, "VMInfo/Users: Found %ld sessions\n", cSessions);
 
     PVBOXSERVICEVMINFOPROC  paProcs;
     DWORD                   cProcs;
@@ -538,38 +545,96 @@ int VBoxServiceVMInfoWinWriteUsers(char **ppszUserList, uint32_t *pcUsersInList)
     if (RT_FAILURE(rc))
     {
         if (rc == VERR_NO_MEMORY)
-            VBoxServiceVerbose(3, "VMInfo/Users: Not enough memory to enumerate processes for a session!\n");
+            VBoxServiceError("VMInfo/Users: Not enough memory to enumerate processes for a session!\n");
         else
             VBoxServiceError("VMInfo/Users: Failed to enumerate processes for a session, rc=%Rrc\n", rc);
     }
     else
     {
-        *pcUsersInList = 0;
-        for (ULONG i = 0; i < cSession; i++)
+        PVBOXSERVICEVMINFOUSER pUserInfo;
+        pUserInfo = (PVBOXSERVICEVMINFOUSER)RTMemAllocZ(cSessions * sizeof(VBOXSERVICEVMINFOUSER) + 1);
+        if (!pUserInfo)
+            VBoxServiceError("VMInfo/Users: Not enough memory to store enumerated users!\n");
+        else
         {
-            VBOXSERVICEVMINFOUSER UserInfo;
-            if (   VBoxServiceVMInfoWinIsLoggedIn(&UserInfo, &paSessions[i])
-                && VBoxServiceVMInfoWinSessionHasProcesses(&paSessions[i], paProcs, cProcs))
+            ULONG cUniqueUsers = 0;
+            for (ULONG i = 0; i < cSessions; i++)
             {
-                if (*pcUsersInList > 0)
+                VBOXSERVICEVMINFOUSER UserInfo;
+                if (VBoxServiceVMInfoWinIsLoggedIn(&UserInfo, &paSessions[i]))
                 {
-                    rc = RTStrAAppend(ppszUserList, ",");
+                    uint32_t cSessionProcs = VBoxServiceVMInfoWinSessionHasProcesses(&paSessions[i], paProcs, cProcs);
+                    if (!cSessionProcs)
+                        continue;
+
+                    bool fFoundUser = false;
+                    for (ULONG i = 0; i < cUniqueUsers; i++)
+                    {
+                        if (   !wcscmp(UserInfo.wszUser, pUserInfo[i].wszUser)
+                            && !wcscmp(UserInfo.wszLogonDomain, pUserInfo[i].wszLogonDomain)
+                            && !wcscmp(UserInfo.wszAuthenticationPackage, pUserInfo[i].wszAuthenticationPackage))
+                        {
+                            /* If the process-per-user count was higher than 0 before but another session was
+                             * was detected which also belongs to this user but has no assigned processes anymore,
+                             * we detected a stale session. */
+                            if (   pUserInfo[i].ulNumProcs > 0
+                                && !cSessionProcs)
+                            {
+                                VBoxServiceVerbose(3, "VMInfo/Users: Stale session for user=%ls detected! Old processes: %u, new: %u\n",
+                                                   pUserInfo[i].wszUser, pUserInfo[i].ulNumProcs, cSessionProcs);
+                            }
+
+                            VBoxServiceVerbose(4, "VMInfo/Users: Updating user=%ls to %u processes\n",
+                                               UserInfo.wszUser, cSessionProcs);
+
+                            pUserInfo[i].ulNumProcs = cSessionProcs;
+                            fFoundUser = true;
+                            break;
+                        }
+                    }
+
+                    if (!fFoundUser)
+                    {
+                        VBoxServiceVerbose(4, "VMInfo/Users: Adding new user=%ls with %u processes\n",
+                                           UserInfo.wszUser, cSessionProcs);
+
+                        memcpy(&pUserInfo[cUniqueUsers], &UserInfo, sizeof(VBOXSERVICEVMINFOUSER));
+                        pUserInfo[cUniqueUsers++].ulNumProcs = cSessionProcs;
+                        Assert(cUniqueUsers <= cSessions);
+                    }
+                }
+            }
+
+            VBoxServiceVerbose(3, "VMInfo/Users: Found %u unique logged-in user(s) with processes\n",
+                               cUniqueUsers);
+
+            *pcUsersInList = 0;
+            for (ULONG i = 0; i < cUniqueUsers; i++)
+            {
+                if (pUserInfo[i].ulNumProcs)
+                {
+                    if (*pcUsersInList > 0)
+                    {
+                        rc = RTStrAAppend(ppszUserList, ",");
+                        AssertRCBreakStmt(rc, RTStrFree(*ppszUserList));
+                    }
+
+                    *pcUsersInList += 1;
+
+                    char *pszTemp;
+                    int rc2 = RTUtf16ToUtf8(pUserInfo[i].wszUser, &pszTemp);
+                    if (RT_SUCCESS(rc2))
+                    {
+                        rc = RTStrAAppend(ppszUserList, pszTemp);
+                        RTMemFree(pszTemp);
+                    }
+                    else
+                        rc = RTStrAAppend(ppszUserList, "<string-conversion-error>");
                     AssertRCBreakStmt(rc, RTStrFree(*ppszUserList));
                 }
-
-                *pcUsersInList += 1;
-
-                char *pszTemp;
-                int rc2 = RTUtf16ToUtf8(UserInfo.wszUser, &pszTemp);
-                if (RT_SUCCESS(rc2))
-                {
-                    rc = RTStrAAppend(ppszUserList, pszTemp);
-                    RTMemFree(pszTemp);
-                }
-                else
-                    rc = RTStrAAppend(ppszUserList, "<string-conversion-error>");
-                AssertRCBreakStmt(rc, RTStrFree(*ppszUserList));
             }
+
+            RTMemFree(pUserInfo);
         }
         VBoxServiceVMInfoWinProcessesFree(paProcs);
     }
