@@ -1,4 +1,4 @@
-/* $Id: VBoxManageMisc.cpp 38191 2011-07-26 17:02:38Z vboxsync $ */
+/* $Id: VBoxManageMisc.cpp $ */
 /** @file
  * VBoxManage - VirtualBox's command-line interface.
  */
@@ -1070,9 +1070,9 @@ int handleExtPack(HandlerArg *a)
             return errorSyntax(USAGE_EXTPACK, "No extension pack name was given to \"extpack install\"");
 
         char szPath[RTPATH_MAX];
-        int vrc = RTPathAbs(a->argv[1], szPath, sizeof(szPath));
+        int vrc = RTPathAbs(pszName, szPath, sizeof(szPath));
         if (RT_FAILURE(vrc))
-            return RTMsgErrorExit(RTEXITCODE_FAILURE, "RTPathAbs(%s,,) failed with rc=%Rrc", a->argv[1], vrc);
+            return RTMsgErrorExit(RTEXITCODE_FAILURE, "RTPathAbs(%s,,) failed with rc=%Rrc", pszName, vrc);
 
         Bstr bstrTarball(szPath);
         Bstr bstrName;
